@@ -2,7 +2,7 @@ use crate::{
     CredentialProtocolMessage, EntityCredential, EntityError, Holder, PresentationFinishedMessage,
     PresentationManifest, Profile,
 };
-use async_trait::async_trait;
+use ockam_core::async_trait;
 use ockam_core::compat::{boxed::Box, string::String, vec::Vec};
 use ockam_core::{Address, Result, Route, Routed, Worker};
 use ockam_node::Context;
@@ -89,11 +89,14 @@ impl Worker for PresenterWorker {
                         revealed,
                     };
 
-                    let presentation = self.profile.create_credential_presentation(
-                        self.credential.bbs_credential(),
-                        &manifest,
-                        request_id,
-                    )?;
+                    let presentation = self
+                        .profile
+                        .create_credential_presentation(
+                            self.credential.bbs_credential(),
+                            &manifest,
+                            request_id,
+                        )
+                        .await?;
 
                     ctx.send(
                         route,
